@@ -70,7 +70,7 @@ if RUBY_VERSION < "2.2"
 end
 
 class FastImage
-  attr_reader :size, :type, :content_length, :orientation
+  attr_reader :size, :type, :content_length, :orientation, :animated
 
   attr_reader :bytes_read
 
@@ -179,6 +179,34 @@ class FastImage
     new(uri, options.merge(:type_only=>true)).type
   end
 
+  # Returns a boolean value indicating the image is animated.
+  # It will return nil if the image could not be fetched, or if the image type was not recognised.
+  #
+  # By default there is a timeout of 2 seconds for opening and reading from a remote server.
+  # This can be changed by passing a :timeout => number_of_seconds in the options.
+  #
+  # If you wish FastImage to raise if it cannot find the type of the image for any reason, then pass
+  # :raise_on_failure => true in the options.
+  #
+  # === Example
+  #
+  #   require 'fastimage'
+  #
+  #   FastImage.animated?("test/fixtures/test.gif")
+  #   => false
+  #   FastImage.animated?("test/fixtures/animated.gif")
+  #   => true
+  #
+  # === Supported options
+  # [:timeout]
+  #   Overrides the default timeout of 2 seconds.  Applies both to reading from and opening the http connection.
+  # [:raise_on_failure]
+  #   If set to true causes an exception to be raised if the image type cannot be found for any reason.
+  #
+  def self.animated?(uri, options={})
+    new(uri, options.merge(:animated_only=>true)).animated
+  end
+
   def initialize(uri, options={})
     @uri = uri
     @options = {
@@ -189,7 +217,13 @@ class FastImage
       :http_header      => {}
     }.merge(options)
 
-    @property = @options[:type_only] ? :type : :size
+    @property = if @options[:animated_only]
+      :animated
+    elsif @options[:type_only]
+      :type
+    else
+      :size
+    end
 
     @type, @state = nil
 
@@ -248,7 +282,7 @@ class FastImage
     begin
       URI(location)
     rescue URI::InvalidURIError
-      URI.escape(location)
+      ::URI::DEFAULT_PARSER.escape(location)
     else
       location
     end
@@ -262,7 +296,10 @@ class FastImage
       if res.is_a?(Net::HTTPRedirection) && @redirect_count < 4
         @redirect_count += 1
         begin
-          @parsed_uri = URI.join(@parsed_uri, escaped_location(res['Location']))
+          location = res['Location']
+          raise ImageFetchFailure if location.nil? || location.empty?
+
+          @parsed_uri = URI.join(@parsed_uri, escaped_location(location))
         rescue URI::InvalidURIError
         else
           fetch_using_http_from_parsed_uri
@@ -369,7 +406,7 @@ class FastImage
 
     begin
       result = send("parse_#{@property}")
-      if result
+      if result != nil
         # extract exif orientation if it was found
         if @property == :size && result.size == 3
           @orientation = result.pop
@@ -391,6 +428,11 @@ class FastImage
     send("parse_size_for_#{@type}")
   end
 
+  def parse_animated
+    @type = parse_type unless @type
+    @type == :gif ? send("parse_animated_for_#{@type}") : nil
+  end
+
   def fetch_using_base64(uri)
     data = uri.split(',')[1]
     decoded = Base64.decode64(data)
@@ -502,7 +544,7 @@ class FastImage
     when "RI"
       :webp if @stream.peek(12)[8..11] == "WEBP"
     when "<s"
-      :svg
+      :svg if @stream.peek(4) == "<svg"
     when /<[?!]/
       # Peek 10 more chars each time, and if end of file is reached just raise
       # unknown. We assume the <svg tag cannot be within 10 chars of the end of
@@ -524,8 +566,76 @@ class FastImage
   end
   alias_method :parse_size_for_cur, :parse_size_for_ico
 
+  class Gif # :nodoc:
+    def initialize(stream)
+      @stream = stream
+    end
+
+    def width_and_height
+      @stream.read(11)[6..10].unpack('SS')
+    end
+
+    # Checks if a delay between frames exists and if it does, then the GIFs is
+    # animated
+    def animated?
+      frames = 0
+
+      # "GIF" + version (3) + width (2) + height (2)
+      @stream.skip(10)
+
+      # fields (1) + bg color (1) + pixel ratio (1)
+      fields = @stream.read(3).unpack("CCC")[0]
+      if fields & 0x80 # Global Color Table
+        # 2 * (depth + 1) colors, each occupying 3 bytes (RGB)
+        @stream.skip(3 * 2 ** ((fields & 0x7) + 1))
+      end
+
+      loop do
+        block_type = @stream.read(1).unpack("C")[0]
+
+        if block_type == 0x21 # Graphic Control Extension
+          # extension type (1) + size (1)
+          size = @stream.read(2).unpack("CC")[1]
+          @stream.skip(size)
+          skip_sub_blocks
+        elsif block_type == 0x2C # Image Descriptor
+          frames += 1
+          return true if frames > 1
+
+          # left position (2) + top position (2) + width (2) + height (2) + fields (1)
+          fields = @stream.read(9).unpack("SSSSC")[4]
+          if fields & 0x80 != 0 # Local Color Table
+            # 2 * (depth + 1) colors, each occupying 3 bytes (RGB)
+            @stream.skip(3 * 2 ** ((fields & 0x7) + 1))
+          end
+
+          @stream.skip(1) # LZW min code size (1)
+          skip_sub_blocks
+        else
+          break # unrecognized block
+        end
+      end
+
+      false
+    end
+
+    private
+
+    def skip_sub_blocks
+      loop do
+        size = @stream.read(1).unpack("C")[0]
+        if size == 0
+          break
+        else
+          @stream.skip(size)
+        end
+      end
+    end
+  end
+
   def parse_size_for_gif
-    @stream.read(11)[6..10].unpack('SS')
+    gif = Gif.new(@stream)
+    gif.width_and_height
   end
 
   def parse_size_for_png
@@ -785,4 +895,9 @@ class FastImage
     svg = Svg.new(@stream)
     svg.width_and_height
   end
+
+  def parse_animated_for_gif
+    gif = Gif.new(@stream)
+    gif.animated?
+  end
 end
