@@ -14,6 +14,7 @@ GoodFixtures = {
   "test_coreheader.bmp"=>[:bmp, [40, 27]],
   "test_v5header.bmp"=>[:bmp, [40, 27]],
   "test.gif"=>[:gif, [17, 32]],
+  "animated.gif"=>[:gif, [400, 400]],
   "test.jpg"=>[:jpeg, [882, 470]],
   "test.png"=>[:png, [30, 20]],
   "test2.jpg"=>[:jpeg, [250, 188]],
@@ -44,6 +45,7 @@ BadFixtures = [
   "faulty.jpg",
   "test_rgb.ct",
   "test.xml",
+  "test2.xml",
   "a.CR2",
   "a.CRW"
 ]
@@ -55,7 +57,7 @@ TestUrl = "http://example.nowhere/"
 
 # this image fetch allows me to really test that fastimage is truly fast
 # but it's not ideal relying on external resources and connectivity speed
-LargeImage = "http://upload.wikimedia.org/wikipedia/commons/b/b4/Mardin_1350660_1350692_33_images.jpg"
+LargeImage = "https://upload.wikimedia.org/wikipedia/commons/b/b4/Mardin_1350660_1350692_33_images.jpg"
 LargeImageInfo = [:jpeg, [9545, 6623]]
 LargeImageFetchLimit = 2  # seconds
 
@@ -102,6 +104,12 @@ class FastImageTest < Test::Unit::TestCase
     end
   end
 
+  def test_should_report_animated_correctly
+    assert_equal nil, FastImage.animated?(TestUrl + "test.png")
+    assert_equal false, FastImage.animated?(TestUrl + "test.gif")
+    assert_equal true, FastImage.animated?(TestUrl + "animated.gif")
+  end
+
   def test_should_return_nil_on_fetch_failure
     assert_nil FastImage.size(TestUrl + "does_not_exist")
   end
@@ -150,9 +158,11 @@ class FastImageTest < Test::Unit::TestCase
     end
   end
 
-  def test_should_raise_unknown_image_typ_when_file_is_non_svg_xml
-    assert_raises(FastImage::UnknownImageType) do
-      FastImage.size(TestUrl + "test.xml", :raise_on_failure=>true)
+  def test_should_raise_unknown_image_type_when_file_is_non_svg_xml
+    ["test.xml", "test2.xml"].each do |fn|
+      assert_raises(FastImage::UnknownImageType) do
+        FastImage.size(TestUrl + fn, :raise_on_failure=>true)
+      end
     end
   end
 
@@ -290,6 +300,13 @@ class FastImageTest < Test::Unit::TestCase
     assert_equal GoodFixtures[GoodFixtures.keys.first][1], FastImage.size(TestUrl, :raise_on_failure=>true)
   end
 
+  def test_should_handle_permanent_redirect_with_missing_location
+    register_redirect(TestUrl, nil)
+    assert_raises(FastImage::ImageFetchFailure) do
+      FastImage.size(TestUrl, :raise_on_failure=>true)
+    end
+  end
+
   def register_redirect(from, to)
     resp = Net::HTTPMovedPermanently.new(1.0, 302, "Moved")
     resp['Location'] = to
